/**
 * @file hal.c
 * @author Shinichiro Nakamura
 * @copyright
 * ===============================================================
 * Natural Tiny Basic (NT-Basic) - A tiny BASIC interpreter
 * Version 0.3.0
 * ===============================================================
 * Copyright (c) 2012-2014 Shinichiro Nakamura
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * ===============================================================
 */

#include "hal.h"

#include <stdio.h>  /* See comments in the functions. */

int hal_init(hal_t *handle, void *extobj)
{
  /*
   * This port uses general libc function.
   * So there is no any initialization code in this port.
   * But you need to change this implementation for your embedded system.
   * (e.g.) Initialize I/O port registers.
   */
  handle->extobj = extobj;
  return 0;
}

int hal_getc(hal_t *handle)
{
  /*
   * This port uses general libc function.
   * But you need to change this implementation for your embedded system.
   * (e.g.) Input from a user input device.
   */
  return getc(stdin);
}

int hal_putc(hal_t *handle, int c)
{
  /*
   * This port uses general libc function.
   * But you need to change this implementation for your embedded system.
   * (e.g.) Output to a LCD display.
   */
  return putc(c, stdout);
}

int hal_puts(hal_t *handle, const char *s)
{
  char *p = (char *)s;
  while (*p) {
    hal_putc(handle, *p);
    p++;
  }
  return 0;
}

char *hal_gets(hal_t *handle, char *s, int size)
{
  char *p = s;
  int i;
  for (i = 0; i < size; i++) {
    int c = hal_getc(handle);
    if ((c == '\0') || (c == '\r') || (c == '\n')) {
      break;
    }
    *p = c;
    p++;
  }
  *p = '\0';
  return s;
}

int hal_read(hal_t *handle, const unsigned int port, unsigned int *data)
{
  /*
   * This port uses general libc function.
   * But you need to change this implementation for your embedded system.
   * (e.g.) Read from a user input device.
   */
  switch (port) {
    case 0:
      printf("HAL:INPUT(PORT=%d)\n", port);
      *data = hal_getc(handle);
      return 0;
    default:
      printf("HAL:INPUT(PORT=%d, DATA=0x%08X)\n", port, *data);
      *data = (port * 100) + (port * 10);
      return 0;
  }
  return -1;
}

int hal_write(hal_t *handle, const unsigned int port, const unsigned int data)
{
  /*
   * This port uses general libc function.
   * But you need to change this implementation for your embedded system.
   * (e.g.) Output to a LCD display.
   */
  switch (port) {
    case 0:
      printf("HAL:OUTPUT(PORT=%d)\n", port);
      hal_putc(handle, data);
      return 0;
    default:
      printf("HAL:OUTPUT(PORT=%d, DATA=0x%08X)\n", port, data);
      /* Do nothing */
      return 0;
  }
  return -1;
}

